@@ -1,5 +1,8 @@
 #!/usr/bin/env python3
-from aws_cdk import core as cdk
+try:
+    from aws_cdk import core as cdk
+except ImportError:
+    import aws_cdk as cdk
 from stacks.chaliceapp import ChaliceApp
 
 app = cdk.App()
